@@ -32,6 +32,7 @@ enum {
     OPT_WINDOW_BORDERLESS,
     OPT_MAX_FPS,
     OPT_LOCK_VIDEO_ORIENTATION,
+    OPT_DISPLAY,
     OPT_DISPLAY_ID,
     OPT_ROTATION,
     OPT_RENDER_DRIVER,
@@ -76,9 +77,19 @@ enum {
     OPT_NO_VIDEO,
     OPT_NO_AUDIO_PLAYBACK,
     OPT_NO_VIDEO_PLAYBACK,
+    OPT_VIDEO_SOURCE,
     OPT_AUDIO_SOURCE,
     OPT_KILL_ADB_ON_CLOSE,
     OPT_TIME_LIMIT,
+    OPT_PAUSE_ON_EXIT,
+    OPT_LIST_CAMERAS,
+    OPT_LIST_CAMERA_SIZES,
+    OPT_CAMERA_ID,
+    OPT_CAMERA_SIZE,
+    OPT_CAMERA_FACING,
+    OPT_CAMERA_AR,
+    OPT_CAMERA_FPS,
+    OPT_CAMERA_HIGH_SPEED,
 };
 
 struct sc_option {
@@ -195,6 +206,51 @@ static const struct sc_option options[] = {
         .longopt = "bit-rate",
         .argdesc = "value",
     },
+    {
+        .longopt_id = OPT_CAMERA_AR,
+        .longopt = "camera-ar",
+        .argdesc = "ar",
+        .text = "Select the camera size by its aspect ratio (+/- 10%).\n"
+                "Possible values are \"sensor\" (use the camera sensor aspect "
+                "ratio), \"<num>:<den>\" (e.g. \"4:3\") or \"<value>\" (e.g. "
+                "\"1.6\")."
+    },
+    {
+        .longopt_id = OPT_CAMERA_ID,
+        .longopt = "camera-id",
+        .argdesc = "id",
+        .text = "Specify the device camera id to mirror.\n"
+                "The available camera ids can be listed by:\n"
+                "    scrcpy --list-cameras",
+    },
+    {
+        .longopt_id = OPT_CAMERA_FACING,
+        .longopt = "camera-facing",
+        .argdesc = "facing",
+        .text = "Select the device camera by its facing direction.\n"
+                "Possible values are \"front\", \"back\" and \"external\".",
+    },
+    {
+        .longopt_id = OPT_CAMERA_HIGH_SPEED,
+        .longopt = "camera-high-speed",
+        .text = "Enable high-speed camera capture mode.\n"
+                "This mode is restricted to specific resolutions and frame "
+                "rates, listed by --list-camera-sizes.",
+    },
+    {
+        .longopt_id = OPT_CAMERA_SIZE,
+        .longopt = "camera-size",
+        .argdesc = "<width>x<height>",
+        .text = "Specify an explicit camera capture size.",
+    },
+    {
+        .longopt_id = OPT_CAMERA_FPS,
+        .longopt = "camera-fps",
+        .argdesc = "value",
+        .text = "Specify the camera capture frame rate.\n"
+                "If not specified, Android's default frame rate (30 fps) is "
+                "used.",
+    },
     {
         // Not really deprecated (--codec has never been released), but without
         // declaring an explicit --codec option, getopt_long() partial matching
@@ -231,9 +287,15 @@ static const struct sc_option options[] = {
         .text = "Disable screensaver while scrcpy is running.",
     },
     {
-        .longopt_id = OPT_DISPLAY_ID,
+        // deprecated
+        .longopt_id = OPT_DISPLAY,
         .longopt = "display",
         .argdesc = "id",
+    },
+    {
+        .longopt_id = OPT_DISPLAY_ID,
+        .longopt = "display-id",
+        .argdesc = "id",
         .text = "Specify the device display id to mirror.\n"
                 "The available display ids can be listed by:\n"
                 "    scrcpy --list-displays\n"
@@ -312,6 +374,16 @@ static const struct sc_option options[] = {
                 "This is a workaround for some devices not behaving as "
                 "expected when setting the device clipboard programmatically.",
     },
+    {
+        .longopt_id = OPT_LIST_CAMERAS,
+        .longopt = "list-cameras",
+        .text = "List device cameras.",
+    },
+    {
+        .longopt_id = OPT_LIST_CAMERA_SIZES,
+        .longopt = "list-camera-sizes",
+        .text = "List the valid camera capture sizes.",
+    },
     {
         .longopt_id = OPT_LIST_DISPLAYS,
         .longopt = "list-displays",
@@ -463,6 +535,20 @@ static const struct sc_option options[] = {
                 "Default is " STR(DEFAULT_LOCAL_PORT_RANGE_FIRST) ":"
                               STR(DEFAULT_LOCAL_PORT_RANGE_LAST) ".",
     },
+    {
+        .longopt_id = OPT_PAUSE_ON_EXIT,
+        .longopt = "pause-on-exit",
+        .argdesc = "mode",
+        .optional_arg = true,
+        .text = "Configure pause on exit. Possible values are \"true\" (always "
+                "pause on exit), \"false\" (never pause on exit) and "
+                "\"if-error\" (pause only if an error occured).\n"
+                "This is useful to prevent the terminal window from "
+                "automatically closing, so that error messages can be read.\n"
+                "Default is \"false\".\n"
+                "Passing the option without argument is equivalent to passing "
+                "\"true\".",
+    },
     {
         .longopt_id = OPT_POWER_OFF_ON_CLOSE,
         .longopt = "power-off-on-close",
@@ -669,6 +755,14 @@ static const struct sc_option options[] = {
                 "codec provided by --video-codec).\n"
                 "The available encoders can be listed by --list-encoders.",
     },
+    {
+        .longopt_id = OPT_VIDEO_SOURCE,
+        .longopt = "video-source",
+        .argdesc = "source",
+        .text = "Select the video source (display or camera).\n"
+                "Camera mirroring requires Android 12+.\n"
+                "Default is display.",
+    },
     {
         .shortopt = 'w',
         .longopt = "stay-awake",
@@ -1070,7 +1164,7 @@ print_shortcut(const struct sc_shortcut *shortcut, unsigned cols) {
     while (shortcut->shortcuts[i]) {
         printf("    %s\n", shortcut->shortcuts[i]);
         ++i;
-    };
+    }
 
     char *text = sc_str_wrap_lines(shortcut->text, cols, 8);
     if (!text) {
@@ -1189,9 +1283,9 @@ parse_integer_arg(const char *s, long *out, bool accept_suffix, long min,
 }
 
 static size_t
-parse_integers_arg(const char *s, size_t max_items, long *out, long min,
-                   long max, const char *name) {
-    size_t count = sc_str_parse_integers(s, ':', max_items, out);
+parse_integers_arg(const char *s, const char sep, size_t max_items, long *out,
+                   long min, long max, const char *name) {
+    size_t count = sc_str_parse_integers(s, sep, max_items, out);
     if (!count) {
         LOGE("Could not parse %s: %s", name, s);
         return 0;
@@ -1238,7 +1332,7 @@ parse_max_size(const char *s, uint16_t *max_size) {
 static bool
 parse_max_fps(const char *s, uint16_t *max_fps) {
     long value;
-    bool ok = parse_integer_arg(s, &value, false, 0, 1000, "max fps");
+    bool ok = parse_integer_arg(s, &value, false, 0, 0xFFFF, "max fps");
     if (!ok) {
         return false;
     }
@@ -1347,7 +1441,7 @@ parse_window_dimension(const char *s, uint16_t *dimension) {
 static bool
 parse_port_range(const char *s, struct sc_port_range *port_range) {
     long values[2];
-    size_t count = parse_integers_arg(s, 2, values, 0, 0xFFFF, "port");
+    size_t count = parse_integers_arg(s, ':', 2, values, 0, 0xFFFF, "port");
     if (!count) {
         return false;
     }
@@ -1609,6 +1703,22 @@ parse_audio_codec(const char *optarg, enum sc_codec *codec) {
     return false;
 }
 
+static bool
+parse_video_source(const char *optarg, enum sc_video_source *source) {
+    if (!strcmp(optarg, "display")) {
+        *source = SC_VIDEO_SOURCE_DISPLAY;
+        return true;
+    }
+
+    if (!strcmp(optarg, "camera")) {
+        *source = SC_VIDEO_SOURCE_CAMERA;
+        return true;
+    }
+
+    LOGE("Unsupported video source: %s (expected display or camera)", optarg);
+    return false;
+}
+
 static bool
 parse_audio_source(const char *optarg, enum sc_audio_source *source) {
     if (!strcmp(optarg, "mic")) {
@@ -1625,6 +1735,46 @@ parse_audio_source(const char *optarg, enum sc_audio_source *source) {
     return false;
 }
 
+static bool
+parse_camera_facing(const char *optarg, enum sc_camera_facing *facing) {
+    if (!strcmp(optarg, "front")) {
+        *facing = SC_CAMERA_FACING_FRONT;
+        return true;
+    }
+
+    if (!strcmp(optarg, "back")) {
+        *facing = SC_CAMERA_FACING_BACK;
+        return true;
+    }
+
+    if (!strcmp(optarg, "external")) {
+        *facing = SC_CAMERA_FACING_EXTERNAL;
+        return true;
+    }
+
+    if (*optarg == '\0') {
+        // Empty string is a valid value (equivalent to not passing the option)
+        *facing = SC_CAMERA_FACING_ANY;
+        return true;
+    }
+
+    LOGE("Unsupported camera facing: %s (expected front, back or external)",
+         optarg);
+    return false;
+}
+
+static bool
+parse_camera_fps(const char *s, uint16_t *camera_fps) {
+    long value;
+    bool ok = parse_integer_arg(s, &value, false, 0, 0xFFFF, "camera fps");
+    if (!ok) {
+        return false;
+    }
+
+    *camera_fps = (uint16_t) value;
+    return true;
+}
+
 static bool
 parse_time_limit(const char *s, sc_tick *tick) {
     long value;
@@ -1637,6 +1787,29 @@ parse_time_limit(const char *s, sc_tick *tick) {
     return true;
 }
 
+static bool
+parse_pause_on_exit(const char *s, enum sc_pause_on_exit *pause_on_exit) {
+    if (!s || !strcmp(s, "true")) {
+        *pause_on_exit = SC_PAUSE_ON_EXIT_TRUE;
+        return true;
+    }
+
+    if (!strcmp(s, "false")) {
+        *pause_on_exit = SC_PAUSE_ON_EXIT_FALSE;
+        return true;
+    }
+
+    if (!strcmp(s, "if-error")) {
+        *pause_on_exit = SC_PAUSE_ON_EXIT_IF_ERROR;
+        return true;
+    }
+
+    LOGE("Unsupported pause on exit mode: %s "
+         "(expected true, false or if-error)", optarg);
+    return false;
+
+}
+
 static bool
 parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
                        const char *optstring, const struct option *longopts) {
@@ -1664,6 +1837,9 @@ parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
             case OPT_CROP:
                 opts->crop = optarg;
                 break;
+            case OPT_DISPLAY:
+                LOGW("--display is deprecated, use --display-id instead.");
+                // fall through
             case OPT_DISPLAY_ID:
                 if (!parse_display_id(optarg, &opts->display_id)) {
                     return false;
@@ -1945,10 +2121,16 @@ parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
                 return false;
 #endif
             case OPT_LIST_ENCODERS:
-                opts->list_encoders = true;
+                opts->list |= SC_OPTION_LIST_ENCODERS;
                 break;
             case OPT_LIST_DISPLAYS:
-                opts->list_displays = true;
+                opts->list |= SC_OPTION_LIST_DISPLAYS;
+                break;
+            case OPT_LIST_CAMERAS:
+                opts->list |= SC_OPTION_LIST_CAMERAS;
+                break;
+            case OPT_LIST_CAMERA_SIZES:
+                opts->list |= SC_OPTION_LIST_CAMERA_SIZES;
                 break;
             case OPT_REQUIRE_AUDIO:
                 opts->require_audio = true;
@@ -1964,6 +2146,11 @@ parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
                     return false;
                 }
                 break;
+            case OPT_VIDEO_SOURCE:
+                if (!parse_video_source(optarg, &opts->video_source)) {
+                    return false;
+                }
+                break;
             case OPT_AUDIO_SOURCE:
                 if (!parse_audio_source(optarg, &opts->audio_source)) {
                     return false;
@@ -1977,6 +2164,33 @@ parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
                     return false;
                 }
                 break;
+            case OPT_PAUSE_ON_EXIT:
+                if (!parse_pause_on_exit(optarg, &args->pause_on_exit)) {
+                    return false;
+                }
+                break;
+            case OPT_CAMERA_AR:
+                opts->camera_ar = optarg;
+                break;
+            case OPT_CAMERA_ID:
+                opts->camera_id = optarg;
+                break;
+            case OPT_CAMERA_SIZE:
+                opts->camera_size = optarg;
+                break;
+            case OPT_CAMERA_FACING:
+                if (!parse_camera_facing(optarg, &opts->camera_facing)) {
+                    return false;
+                }
+                break;
+            case OPT_CAMERA_FPS:
+                if (!parse_camera_fps(optarg, &opts->camera_fps)) {
+                    return false;
+                }
+                break;
+            case OPT_CAMERA_HIGH_SPEED:
+                opts->camera_high_speed = true;
+                break;
             default:
                 // getopt prints the error message on stderr
                 return false;
@@ -2022,12 +2236,6 @@ parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
         opts->audio_playback = false;
     }
 
-    if (!opts->video_playback && !otg) {
-        // If video playback is disabled and OTG are disabled, then there is
-        // no way to control the device.
-        opts->control = false;
-    }
-
     if (opts->video && !opts->video_playback && !opts->record_filename
             && !v4l2) {
         LOGI("No video playback, no recording, no V4L2 sink: video disabled");
@@ -2076,6 +2284,58 @@ parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
         opts->force_adb_forward = true;
     }
 
+    if (opts->video_source == SC_VIDEO_SOURCE_CAMERA) {
+        if (opts->display_id) {
+            LOGE("--display-id is only available with --video-source=display");
+            return false;
+        }
+
+        if (opts->camera_id && opts->camera_facing != SC_CAMERA_FACING_ANY) {
+            LOGE("Could not specify both --camera-id and --camera-facing");
+            return false;
+        }
+
+        if (opts->camera_size) {
+            if (opts->max_size) {
+                LOGE("Could not specify both --camera-size and -m/--max-size");
+                return false;
+            }
+
+            if (opts->camera_ar) {
+                LOGE("Could not specify both --camera-size and --camera-ar");
+                return false;
+            }
+        }
+
+        if (opts->camera_high_speed && !opts->camera_fps) {
+            LOGE("--camera-high-speed requires an explicit --camera-fps value");
+            return false;
+        }
+
+        if (opts->control) {
+            LOGI("Camera video source: control disabled");
+            opts->control = false;
+        }
+    } else if (opts->camera_id
+            || opts->camera_ar
+            || opts->camera_facing != SC_CAMERA_FACING_ANY
+            || opts->camera_fps
+            || opts->camera_high_speed
+            || opts->camera_size) {
+        LOGE("Camera options are only available with --video-source=camera");
+        return false;
+    }
+
+    if (opts->audio && opts->audio_source == SC_AUDIO_SOURCE_AUTO) {
+        // Select the audio source according to the video source
+        if (opts->video_source == SC_VIDEO_SOURCE_DISPLAY) {
+            opts->audio_source = SC_AUDIO_SOURCE_OUTPUT;
+        } else {
+            opts->audio_source = SC_AUDIO_SOURCE_MIC;
+            LOGI("Camera video source: microphone audio source selected");
+        }
+    }
+
     if (opts->record_format && !opts->record_filename) {
         LOGE("Record format specified without recording");
         return false;
@@ -2196,6 +2456,37 @@ parse_args_with_getopt(struct scrcpy_cli_args *args, int argc, char *argv[],
     return true;
 }
 
+static enum sc_pause_on_exit
+sc_get_pause_on_exit(int argc, char *argv[]) {
+    // Read arguments backwards so that the last --pause-on-exit is considered
+    // (same behavior as getopt())
+    for (int i = argc - 1; i >= 1; --i) {
+        const char *arg = argv[i];
+        // Starts with "--pause-on-exit"
+        if (!strncmp("--pause-on-exit", arg, 15)) {
+            if (arg[15] == '\0') {
+                // No argument
+                return SC_PAUSE_ON_EXIT_TRUE;
+            }
+            if (arg[15] != '=') {
+                // Invalid parameter, ignore
+                return SC_PAUSE_ON_EXIT_FALSE;
+            }
+            const char *value = &arg[16];
+            if (!strcmp(value, "true")) {
+                return SC_PAUSE_ON_EXIT_TRUE;
+            }
+            if (!strcmp(value, "if-error")) {
+                return SC_PAUSE_ON_EXIT_IF_ERROR;
+            }
+            // Set to false, inclusing when the value is invalid
+            return SC_PAUSE_ON_EXIT_FALSE;
+        }
+    }
+
+    return SC_PAUSE_ON_EXIT_FALSE;
+}
+
 bool
 scrcpy_parse_args(struct scrcpy_cli_args *args, int argc, char *argv[]) {
     struct sc_getopt_adapter adapter;
@@ -2209,5 +2500,11 @@ scrcpy_parse_args(struct scrcpy_cli_args *args, int argc, char *argv[]) {
 
     sc_getopt_adapter_destroy(&adapter);
 
+    if (!ret && args->pause_on_exit == SC_PAUSE_ON_EXIT_FALSE) {
+        // Check if "--pause-on-exit" is present in the arguments list, because
+        // it must be taken into account even if command line parsing failed
+        args->pause_on_exit = sc_get_pause_on_exit(argc, argv);
+    }
+
     return ret;
 }
