@@ -21,8 +21,20 @@ enum sc_record_format {
     SC_RECORD_FORMAT_AUTO,
     SC_RECORD_FORMAT_MP4,
     SC_RECORD_FORMAT_MKV,
+    SC_RECORD_FORMAT_M4A,
+    SC_RECORD_FORMAT_MKA,
+    SC_RECORD_FORMAT_OPUS,
+    SC_RECORD_FORMAT_AAC,
 };
 
+static inline bool
+sc_record_format_is_audio_only(enum sc_record_format fmt) {
+    return fmt == SC_RECORD_FORMAT_M4A
+        || fmt == SC_RECORD_FORMAT_MKA
+        || fmt == SC_RECORD_FORMAT_OPUS
+        || fmt == SC_RECORD_FORMAT_AAC;
+}
+
 enum sc_codec {
     SC_CODEC_H264,
     SC_CODEC_H265,
@@ -32,6 +44,24 @@ enum sc_codec {
     SC_CODEC_RAW,
 };
 
+enum sc_video_source {
+    SC_VIDEO_SOURCE_DISPLAY,
+    SC_VIDEO_SOURCE_CAMERA,
+};
+
+enum sc_audio_source {
+    SC_AUDIO_SOURCE_AUTO, // OUTPUT for video DISPLAY, MIC for video CAMERA
+    SC_AUDIO_SOURCE_OUTPUT,
+    SC_AUDIO_SOURCE_MIC,
+};
+
+enum sc_camera_facing {
+    SC_CAMERA_FACING_ANY,
+    SC_CAMERA_FACING_FRONT,
+    SC_CAMERA_FACING_BACK,
+    SC_CAMERA_FACING_EXTERNAL,
+};
+
 enum sc_lock_video_orientation {
     SC_LOCK_VIDEO_ORIENTATION_UNLOCKED = -1,
     // lock the current orientation when scrcpy starts
@@ -100,15 +130,19 @@ struct scrcpy_options {
     const char *audio_codec_options;
     const char *video_encoder;
     const char *audio_encoder;
-#ifdef HAVE_V4L2
-    const char *v4l2_device;
-#endif
+    const char *camera_id;
+    const char *camera_size;
+    const char *camera_ar;
+    uint16_t camera_fps;
     enum sc_log_level log_level;
     enum sc_codec video_codec;
     enum sc_codec audio_codec;
+    enum sc_video_source video_source;
+    enum sc_audio_source audio_source;
     enum sc_record_format record_format;
     enum sc_keyboard_input_mode keyboard_input_mode;
     enum sc_mouse_input_mode mouse_input_mode;
+    enum sc_camera_facing camera_facing;
     struct sc_port_range port_range;
     uint32_t tunnel_host;
     uint16_t tunnel_port;
@@ -125,8 +159,13 @@ struct scrcpy_options {
     uint16_t window_height;
     uint32_t display_id;
     sc_tick display_buffer;
-    sc_tick v4l2_buffer;
     sc_tick audio_buffer;
+    sc_tick audio_output_buffer;
+    sc_tick time_limit;
+#ifdef HAVE_V4L2
+    const char *v4l2_device;
+    sc_tick v4l2_buffer;
+#endif
 #ifdef HAVE_USB
     bool otg;
 #endif
@@ -134,7 +173,8 @@ struct scrcpy_options {
     bool fullscreen;
     bool always_on_top;
     bool control;
-    bool display;
+    bool video_playback;
+    bool audio_playback;
     bool turn_screen_off;
     enum sc_key_inject_mode key_inject_mode;
     bool window_borderless;
@@ -155,10 +195,16 @@ struct scrcpy_options {
     bool cleanup;
     bool start_fps_counter;
     bool power_on;
+    bool video;
     bool audio;
     bool require_audio;
-    bool list_encoders;
-    bool list_displays;
+    bool kill_adb_on_close;
+    bool camera_high_speed;
+#define SC_OPTION_LIST_ENCODERS 0x1
+#define SC_OPTION_LIST_DISPLAYS 0x2
+#define SC_OPTION_LIST_CAMERAS 0x4
+#define SC_OPTION_LIST_CAMERA_SIZES 0x8
+    uint8_t list;
 };
 
 extern const struct scrcpy_options scrcpy_options_default;
