@@ -28,14 +28,13 @@ public final class Workarounds {
         // not instantiable
     }
 
-    public static void apply(boolean audio) {
+    public static void apply(boolean audio, boolean camera) {
         Workarounds.prepareMainLooper();
 
         boolean mustFillAppInfo = false;
         boolean mustFillBaseContext = false;
         boolean mustFillAppContext = false;
 
-
         if (Build.BRAND.equalsIgnoreCase("meizu")) {
             // Workarounds must be applied for Meizu phones:
             //  - <https://github.com/Genymobile/scrcpy/issues/240>
@@ -65,6 +64,11 @@ public final class Workarounds {
             mustFillAppContext = true;
         }
 
+        if (camera) {
+            mustFillAppInfo = true;
+            mustFillBaseContext = true;
+        }
+
         if (mustFillAppInfo) {
             Workarounds.fillAppInfo();
         }
@@ -154,7 +158,7 @@ public final class Workarounds {
         }
     }
 
-    public static void fillBaseContext() {
+    private static void fillBaseContext() {
         try {
             fillActivityThread();
 
